// This script is modified from the AutoWayPoint from the FPS tut. 
// To use this script, create an empty game object and attach this script to it. You must also create tags 
// for "path1", "path2", "path3", "path4" & "path5"(you can create more if you like, you'll have to edit the script, 
// and provide a tif picture file for your extra path nodes), and set the exposed variable pathID identical to the
// tag selected for the node
// For example: If you have 5 nodes as path1, then you set their tagsas "path1", and edit the variable pathID to "path1"
// Make sure your Gizmos folder is in the same hiearchy as Standard Assets/ProAssets and your tif files are in it.
// tif file names are listed below in OnDrawGizmos() function, edit the string name of the file in Gizmos.DrawIcons()
// if needed.

var connected = Array();
var pathID : String = "path1";
var kLineOfSightCapsuleRadius = 0.25; // the radius for checking line of site 
private var waypoints = Array();

function FindClosest (pos : Vector3, pathName : String) {
	if(pathName == null) { pathName = pathID; }
	var gos : GameObject[];
	gos = GameObject.FindGameObjectsWithTag(pathName);
	// The closer two vectors, the larger the dot product will be.
	var closest : GameObject;
	var closestDistance = 100000.0;
	for (var cur : GameObject in gos) {
		var distance = Vector3.Distance(cur.transform.position, pos);
		if (distance < closestDistance)
		{
			closestDistance = distance;
			closest = cur;
		}
	}
	return closest;
}

@ContextMenu ("Update Waypoints")
function UpdateWaypoints () {
	RebuildWaypointList();
}

function Awake () {
	RebuildWaypointList();
}


// Draw the waypoint pickable gizmo
function OnDrawGizmos () {
	if(gameObject.tag == "path1") { Gizmos.DrawIcon(transform.position, "Waypoint1.tif"); }
	if(gameObject.tag == "path2") { Gizmos.DrawIcon(transform.position, "Waypoint2.tif"); }
	if(gameObject.tag == "path3") { Gizmos.DrawIcon(transform.position, "Waypoint3.tif"); }
	if(gameObject.tag == "path4") { Gizmos.DrawIcon(transform.position, "Waypoint4.tif"); }
	if(gameObject.tag == "path5") { Gizmos.DrawIcon(transform.position, "Waypoint5.tif"); }
}

// Draw the waypoint lines only when you select one of the waypoints
function OnDrawGizmosSelected () {
	if (waypoints.length == 0)
		RebuildWaypointList();
	for (var p : GameObject in connected) {
		if (Physics.Linecast(transform.position, p.transform.position)) {
			Gizmos.color = Color.red;
			Gizmos.DrawLine (transform.position, p.transform.position);
		} else {
			if(p.tag == "path1") { Gizmos.color = Color.green; }
			if(p.tag == "path2") { Gizmos.color = Color.blue; }
			if(p.tag == "path3") { Gizmos.color = Color.yellow; }
			if(p.tag == "path4") { Gizmos.color = Color.cyan; }
			if(p.tag == "path5") { Gizmos.color = Color.magenta; }
			Gizmos.DrawLine (transform.position, p.transform.position);
		}
	}
}

function RebuildWaypointList () {
	var objects : GameObject[];
	objects = GameObject.FindGameObjectsWithTag(pathID);
	waypoints = Array(objects);
	var ctr : int = 0;
	for (var point : GameObject in waypoints) {
		RecalculateConnectedWaypoints();
	}
}

function RecalculateConnectedWaypoints ()
{
	connected = Array();

	for (var other : GameObject in waypoints) {
		// Don't connect to ourselves
		if (other == this)
			continue;
		
		// Do we have a clear line of sight?
		if (!Physics.CheckCapsule(transform.position, other.transform.position, kLineOfSightCapsuleRadius)) {
			connected.Add(other);
		}
	}	
}