// simple ai patrol path script(handles calls to multiple paths in AutoWayPoint)
// To use this script, simply attach to ai bot to provide non-animated movement on a path.
// you will need to edit the exposed variables, for movement/rotation speeds, and edit the 
// string to "path1", "path2", "path3", "path4" & "path5", for whichever oath belongs to the bot.

var moveSpeed : int = 6.0; // speed of ai on path
var rotationSpeed : float = 1.8;
var PathID : String = " "; // initialize to null
var pickNextWaypointDistance = 0.5; // distnce of how close to get to node, before picking next one

function Start()
{
	Patrol();
}

function Patrol() 
{
	var gameobjects : GameObject; 
	gameobjects = GameObject.FindWithTag(PathID);    
	var curWayPoint = gameobjects.GetComponent(AutoWayPoint).FindClosest(transform.position,PathID);
	while (true) 
	{	
		var waypointPosition = curWayPoint.transform.position;
		// Are we close to a waypoint? -> pick the next one!
		if(Vector3.Distance(waypointPosition, transform.position) < pickNextWaypointDistance)
		{
		   curWayPoint = PickNextWaypoint(curWayPoint); 
		}
		// Move towards our target
		MoveTowards(waypointPosition);	
		
		yield;	    
	}
}

function MoveTowards (position : Vector3) 
{
	var targetdirection = position - transform.position;
	targetdirection.y = 0;
	
	
	// Rotate towards the target
	transform.rotation = Quaternion.Slerp (transform.rotation, Quaternion.LookRotation(targetdirection), rotationSpeed * Time.deltaTime);
	transform.eulerAngles = Vector3(0, transform.eulerAngles.y, 0);

	// Modify speed so we slow down when we are not facing the target
	var targetforward = transform.TransformDirection(Vector3.forward);
	var speedModifier = Vector3.Dot(targetforward, targetdirection.normalized);
	speedModifier = Mathf.Clamp01(speedModifier);
	 
	
	// Move the character
	targetdirection = targetforward * moveSpeed * speedModifier;
	GetComponent(CharacterController).SimpleMove(targetdirection);
}

function PickNextWaypoint (currentWaypoint : GameObject) 
{
	// We want to find the waypoint where the character has to turn the least

	// The direction in which we are walking
	var forward = transform.TransformDirection(Vector3.forward);

	// The closer two vectors, the larger the dot product will be.
	var best = currentWaypoint;
	var bestDot = -10.0;
	for (var cur : GameObject in currentWaypoint.GetComponent(AutoWayPoint).connected) 
	{
		var direction = Vector3.Normalize(cur.transform.position - transform.position);
		var dot = Vector3.Dot(direction, forward);
		if (dot > bestDot && cur != currentWaypoint) 
		{
			bestDot = dot;
			best = cur;
		}
	}	
	return best;
}
